class GUIConfig:
    # Text for the Beian link
    ICP_BEIAN_TEXT = "晋ICP备2025058330号"
    
    # Website name and title
    WEBSITE_NAME = "你好👋"
    WEBSITE_TITLE = "医键通词库转换工具"
    BACKEND_VERSION = "2.3.1"
    GUI_VERSION = "2.3.1"

    # Optional Google AdSense snippet to inject into the GUI
    GOOGLE_AD_SNIPPET = """<script async src=\"https://pagead2.googlesyndication.com/pagead/js/adsbygoogle.js?client=ca-pub-5988994930330758\" crossorigin=\"anonymous\"></script>\n<!-- rimetool -->\n<ins class=\"adsbygoogle\" style=\"display:block\" data-ad-client=\"ca-pub-5988994930330758\" data-ad-slot=\"3211445360\" data-ad-format=\"auto\" data-full-width-responsive=\"true\"></ins>\n<script>\n     (adsbygoogle = window.adsbygoogle || []).push({});\n</script>"""
